 
 $(document).ready(function() {
   
   var dataSource = new EditorDataSource({
		columns: [
			{
				property: 'name',
				label: 'Name',
				sortable: true
			},
			{
				property: 'status',
				label: 'Status',
				sortable: true
			},
			{
				property: 'author',
				label: 'Author',
				sortable: true
			},
			{
				property: 'timestamp',
				label: 'Date',
				sortable: true
			}
		],
		
		formatter: function (items) {
            $.each(items, function (index, item) {
              if(item.status == ("Failed")) {
                  item.status = "<span style='color: red'>" + item.status + "</span>";
              
              } else if(item.status == "Awaiting Editorial") {
                  item.status = "<span style='color: blue'>" + item.status + "</span><form method='post' action='article/" + item.id + "/start'><button>Start Editorial</button></form>";
              
              } else if(item.status == "In Editorial") {
                  item.status = "<span style='color: orange'><a href='article/" + item.id + "/edit'>" + item.status + "</a></span><form method='post' action='article/" + item.id + "/finish'><button>Finish Editorial</button></form>";
              } else if(item.status == "In Assembly") {
                  item.status = "<span style='color: green'><a href='article/" + item.id + "/edit'>" + item.status + "</a></span><form method='post' action='article/" + item.id + "/finish'><a href='article/" + item.id + "/pdf'>(PDF Preview)</button></form>";
              }
              
              item.timestamp = new Date(item.timestamp);
              /* item.image = '<img src="' + flickrUrl(item) + '"></a>'; */
              item
            });
        },
        
		delay: 250
	});

   
   $('#editor-grid').datagrid({ dataSource: dataSource, stretchHeight: true })
   
});

var EditorDataSource = function (options) {
	this._formatter = options.formatter;
	this._columns = options.columns;
};

EditorDataSource.prototype = {

	/**
	 * Returns stored column metadata
	 */
	columns: function () {
		return this._columns;
	},

	/**
	 * Called when Datagrid needs data. Logic should check the options parameter
	 * to determine what data to return, then return data by calling the callback.
	 * @param {object} options Options selected in datagrid (ex: {pageIndex:0,pageSize:5,search:'searchterm'})
	 * @param {function} callback To be called with the requested data.
	 */
	data: function (options, callback) {

		var url = 'http://localhost:8080/exist/restxq/boredcat/editor/articles';
		var self = this;

		if (options.search) {

			// Search active.  Add URL parameters for Flickr API.
			url += '&tags=' + options.search;
			url += '&per_page=' + options.pageSize;
			url += '&page=' + (options.pageIndex + 1);

		}

		$.ajax(url).done(function (response) {

			// Prepare data to return to Datagrid
			var data = response.article;
			
			var count = response.total;
			//var startIndex = (response.photos.page - 1) * response.photos.perpage;
			var startIndex = 1;
			var endIndex = 10;
			//var endIndex = startIndex + response.photos.perpage;
			var end = (endIndex > count) ? count : endIndex;
			var pages = 1;
			var page = 1;
			//var pages = response.photos.pages;
			//var page = response.photos.page;
			var start = startIndex + 1;

			// Allow client code to format the data
			if (self._formatter) self._formatter(data);

			// Return data to Datagrid
			callback({ data: data, start: start, end: end, count: count, pages: pages, page: page });

		});
	}
};