xquery version "3.0";

declare namespace trigger = "http://exist-db.org/xquery/trigger";

import module namespace compression = "http://exist-db.org/xquery/compression";
import module namespace sm = "http://exist-db.org/xquery/securitymanager";
import module namespace system = "http://exist-db.org/xquery/system";
import module namespace util = "http://exist-db.org/xquery/util";
import module namespace validation = "http://exist-db.org/xquery/validation";
import module namespace xmldb = "http://exist-db.org/xquery/xmldb";



declare function trigger:after-create-document($uri as xs:anyURI) {
    
    (:
        skip anything that is not a .zip file and files that start with a '.' i.e. from macosx webdav
    :)
    if(not(ends-with($uri, ".zip")) or starts-with(replace($uri, ".*/", ""), "."))then
        util:log("info", ("submission-trigger.xql ignoring file path: ", $uri))
    else
        (: 1. create a temp collection to unpack the uploaded zip to :)
        let $uuid := util:uuid()
        let $temp-collection := xmldb:create-collection("/db/publishing/articles/submission/validation/" || xmldb:get-current-user(), $uuid)
        return
        
            (: 2. unpack the uploaded zip to the temp collection :)
            let $unzipped := compression:unzip(
                util:binary-doc($uri),
                function($path, $data-type, $param) { true() },
                (),
                function($path, $data-type, $param) { xs:anyURI($temp-collection || "/" || $path) },
                ()
            ) return
        
                (: 3. find the xml documents in the unpacked collection :)
                let $doc-uris := xmldb:get-child-resources($temp-collection)[ends-with(., ".xml")] ! concat($temp-collection, "/", .)
                    
                (: 4. validate the unpacked xml documents against the DocBook Schema :)
                let $valid := local:validate-docbook(replace($uri, ".*/", ""), $doc-uris) return
                    
                    (: 5. record that we validated the submission :)
                    let $update :=
                        update insert
                            <article id="{$uuid}" author="{xmldb:get-current-user()}">
                                <submission timestamp="{current-dateTime()}" valid="{$valid}">{$uri}</submission>
                            </article>
                        into doc(xs:anyURI("/db/publishing/articles/status.xml"))/articles
                    return
                
                        (: 6. was the docbook valid? :)
                        if($valid)then
                            
                            (: TODO validate some business rules, e.g. are all figure uris de-referenceable, cross check with all figure images referenced :)
                            
                            
                            (: 6.1. as the article was valid, copy it to the editorial inbox :)
                            let $editorial-inbox-collection := xs:anyURI("/db/publishing/articles/editorial/inbox"),
                            $null := xmldb:copy($temp-collection, $editorial-inbox-collection),
                            $new-coll := xs:anyURI($editorial-inbox-collection || "/" || $uuid),
                            $null := system:as-user("admin", "", (local:chown-r($new-coll, "admin:editors"), local:chmod-r($new-coll, "rwxrwxr-x"))),    (: grant access to editor users, remove access for author :)
                            $null := xmldb:remove($temp-collection)
                            return
                            
                                (: 6.2. record that we copied it to the editorial inbox in the status :)
                                update insert
                                    <editorial-inbox>{concat($editorial-inbox-collection, "/", $uuid)}</editorial-inbox>
                                into doc(xs:anyURI("/db/publishing/articles/status.xml"))/articles/article[submission eq $uri]
                            
                        else()
};

declare
    %private
function local:validate-docbook($name as xs:string, $uris as xs:string*) as xs:boolean {
    
    let $reports := $uris ! validation:validate-report(doc(.), xs:anyURI("/db/publishing/validation/docbook.rng")) return 
        if($reports/status = "invalid")then
            (: report validation failure :)
            let $null := local:report-validation-failure($name, $reports) return
                false()
        else
            true()
};

declare
    %private
function local:report-validation-failure($name, $reports) {
    
    (: log the report:)
    util:log("INFO", ("submission-trigger.xql failed validation for: ", xmldb:get-current-user(), $name, $reports)),
    
    (: email report to author:)
    let $author-email := sm:get-account-metadata(xmldb:get-current-user(), xs:anyURI("http://axschema.org/contact/email"))
    return
        if(exists($author-email))then
            local:send-report($author-email, $name, $reports)
        else()
};

declare
    %private
function local:send-report($destination, $name, $reports) as empty() {
    
    (: TODO email the report to the email address in $destination :)
    ()
};


declare
    %private
function local:chown-r($uri as xs:anyURI, $user) {
    sm:chown($uri, $user)
    ,
    for $child in xmldb:get-child-resources($uri)
    return
        sm:chown(xs:anyURI($uri || "/" || $child), $user)
    ,
    for $sub-coll in xmldb:get-child-collections($uri)
    return
        local:chown-r(xs:anyURI($uri || "/" || $sub-coll), $user)
    
};

declare
    %private
function local:chmod-r($uri as xs:anyURI, $mode) {
    sm:chmod($uri, $mode)
    ,
    for $child in xmldb:get-child-resources($uri)
    return
        sm:chmod(xs:anyURI($uri || "/" || $child), $mode)
    ,
    for $sub-coll in xmldb:get-child-collections($uri)
    return
        local:chmod-r(xs:anyURI($uri || "/" || $sub-coll), $mode)
    
};


(: main: do nothing - this is a trigger :)
()
